@@ -20,7 +20,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 #include <stdio.h>      /* rename() */
@@ -37,7 +37,6 @@
 #include "rb-static-playlist-source.h"
 #include "rb-auto-playlist-source.h"
 #include "rb-play-queue-source.h"
-#include "rb-recorder.h"
 #include "rb-sourcelist.h"
 #include "rb-sourcelist-model.h"
 #include "rb-query-creator.h"
@@ -57,6 +56,7 @@
 
 static void rb_playlist_manager_class_init (RBPlaylistManagerClass *klass);
 static void rb_playlist_manager_init (RBPlaylistManager *mgr);
+static void rb_playlist_manager_dispose (GObject *object);
 static void rb_playlist_manager_finalize (GObject *object);
 static void rb_playlist_manager_set_property (GObject *object,
 					      guint prop_id,
@@ -70,8 +70,6 @@ static void rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 						   RBPlaylistManager *mgr);
-static void rb_playlist_manager_cmd_burn_playlist (GtkAction *action,
-						   RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_playlist (GtkAction *action,
 						  RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
@@ -84,9 +82,6 @@ static void rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
 							     RBPlaylistManager *mgr);
 static void rb_playlist_manager_cmd_queue_playlist (GtkAction *action,
 						    RBPlaylistManager *mgr);
-static void rb_playlist_manager_playlist_entries_changed (GtkTreeModel *entry_view,
-							  RhythmDBEntry *entry,
-							  RBPlaylistManager *mgr);
 
 struct RBPlaylistManagerPrivate
 {
@@ -94,6 +89,8 @@ struct RBPlaylistManagerPrivate
 	RBShell *shell;
 	RBSource *selected_source;
 
+	char *playlists_file;
+
 	RBSourceList *sourcelist;
 
 	GtkActionGroup *actiongroup;
@@ -111,6 +108,7 @@ struct RBPlaylistManagerPrivate
 enum
 {
 	PROP_0,
+	PROP_PLAYLIST_NAME,
 	PROP_SHELL,
 	PROP_SOURCE,
 	PROP_SOURCELIST,
@@ -144,9 +142,6 @@ static GtkActionEntry rb_playlist_manager_actions [] =
 	{ "MusicPlaylistSavePlaylist", GTK_STOCK_SAVE_AS, N_("_Save to File..."), NULL,
 	  N_("Save a playlist to a file"),
 	  G_CALLBACK (rb_playlist_manager_cmd_save_playlist) },
-	{ "MusicPlaylistBurnPlaylist", GTK_STOCK_CDROM, N_("_Create Audio CD..."), NULL,
-	  N_("Create an audio CD from playlist"),
-	  G_CALLBACK (rb_playlist_manager_cmd_burn_playlist) },
 	{ "MusicPlaylistRenamePlaylist", NULL, N_("_Rename"), NULL,
 	  N_("Rename playlist"),
 	  G_CALLBACK (rb_playlist_manager_cmd_rename_playlist) },
@@ -169,11 +164,20 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = rb_playlist_manager_dispose;
 	object_class->finalize = rb_playlist_manager_finalize;
 
 	object_class->set_property = rb_playlist_manager_set_property;
 	object_class->get_property = rb_playlist_manager_get_property;
 
+	g_object_class_install_property (object_class,
+					 PROP_PLAYLIST_NAME,
+                                         g_param_spec_string ("playlists_file",
+                                                              "name",
+                                                              "playlists file",
+                                                              NULL,
+                                                              G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_SOURCE,
 					 g_param_spec_object ("source",
@@ -190,7 +194,6 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 							      RB_TYPE_SHELL,
 							      G_PARAM_READWRITE));
 
-
 	g_object_class_install_property (object_class,
 					 PROP_SOURCELIST,
 					 g_param_spec_object ("sourcelist",
@@ -207,7 +210,7 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
-	
+
 	rb_playlist_manager_signals[PLAYLIST_CREATED] =
 		g_signal_new ("playlist_created",
 			      RB_TYPE_PLAYLIST_MANAGER,
@@ -243,8 +246,8 @@ rb_playlist_manager_class_init (RBPlaylistManagerClass *klass)
 static void
 rb_playlist_manager_init (RBPlaylistManager *mgr)
 {
-	mgr->priv = G_TYPE_INSTANCE_GET_PRIVATE (mgr, 
-						 RB_TYPE_PLAYLIST_MANAGER, 
+	mgr->priv = G_TYPE_INSTANCE_GET_PRIVATE (mgr,
+						 RB_TYPE_PLAYLIST_MANAGER,
 						 RBPlaylistManagerPrivate);
 
 	mgr->priv->saving_mutex = g_mutex_new ();
@@ -261,6 +264,43 @@ rb_playlist_manager_shutdown (RBPlaylistManager *mgr)
 	g_mutex_unlock (mgr->priv->saving_mutex);
 }
 
+static void
+rb_playlist_manager_dispose (GObject *object)
+{
+	RBPlaylistManager *mgr;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_PLAYLIST_MANAGER (object));
+
+	rb_debug ("Disposing playlist manager");
+
+	mgr = RB_PLAYLIST_MANAGER (object);
+
+	g_return_if_fail (mgr->priv != NULL);
+
+	if (mgr->priv->db != NULL) {
+		g_object_unref (mgr->priv->db);
+		mgr->priv->db = NULL;
+	}
+
+	if (mgr->priv->uimanager != NULL) {
+		g_object_unref (mgr->priv->uimanager);
+		mgr->priv->uimanager = NULL;
+	}
+
+	if (mgr->priv->sourcelist != NULL) {
+		g_object_unref (mgr->priv->sourcelist);
+		mgr->priv->sourcelist = NULL;
+	}
+
+	if (mgr->priv->selected_source != NULL) {
+		g_object_unref (mgr->priv->selected_source);
+		mgr->priv->selected_source = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_playlist_manager_parent_class)->dispose (object);
+}
+
 static void
 rb_playlist_manager_finalize (GObject *object)
 {
@@ -277,11 +317,13 @@ rb_playlist_manager_finalize (GObject *object)
 
 	g_mutex_free (mgr->priv->saving_mutex);
 
+	g_free (mgr->priv->playlists_file);
+
 	G_OBJECT_CLASS (rb_playlist_manager_parent_class)->finalize (object);
 }
 
 static void
-rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr, 
+rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr,
 				   GtkUIManager *uimanager)
 {
 	if (mgr->priv->uimanager != NULL) {
@@ -289,8 +331,7 @@ rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr,
 			gtk_ui_manager_remove_action_group (mgr->priv->uimanager,
 							    mgr->priv->actiongroup);
 		}
-		g_object_unref (G_OBJECT (mgr->priv->uimanager));
-		mgr->priv->uimanager = NULL;
+		g_object_unref (mgr->priv->uimanager);
 	}
 
 	mgr->priv->uimanager = uimanager;
@@ -304,36 +345,12 @@ rb_playlist_manager_set_uimanager (RBPlaylistManager *mgr,
 					      rb_playlist_manager_n_actions,
 					      mgr);
 	}
-	
+
 	gtk_ui_manager_insert_action_group (mgr->priv->uimanager,
 					    mgr->priv->actiongroup,
 					    0);
 }
 
-static void
-rb_playlist_manager_playlist_entries_changed (GtkTreeModel *model, RhythmDBEntry *entry, RBPlaylistManager *mgr)
-{
-	int num_tracks;
-	GtkAction *action;
-
-	num_tracks = gtk_tree_model_iter_n_children (model, NULL);
-
-	action = gtk_action_group_get_action (mgr->priv->actiongroup, "MusicPlaylistBurnPlaylist");
-	gtk_action_set_sensitive (action, (num_tracks > 0));
-}
-
-static void
-rb_playlist_manager_playlist_row_inserted_cb (GtkTreeModel *model,
-					      GtkTreePath *path,
-					      GtkTreeIter *iter,
-					      RBPlaylistManager *mgr)
-{
-	RhythmDBEntry *entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
-
-	rb_playlist_manager_playlist_entries_changed (model, entry, mgr);
-}
-
-
 static void
 rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 				RBSource *source)
@@ -347,26 +364,16 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 	gboolean can_rename;
 	GtkAction *action;
 
-	if (mgr->priv->selected_source != NULL) {
-		RhythmDBQueryModel *model;
-
-		g_object_get (G_OBJECT (mgr->priv->selected_source), "query-model", &model, NULL);
-		
-		g_signal_handlers_disconnect_by_func (G_OBJECT (model),
-						      G_CALLBACK (rb_playlist_manager_playlist_entries_changed),
-						      mgr);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (model),
-						      G_CALLBACK (rb_playlist_manager_playlist_row_inserted_cb),
-						      mgr);
-		g_object_unref (G_OBJECT (model));
-	}
-
 	party_mode = rb_shell_get_party_mode (mgr->priv->shell);
 
-	mgr->priv->selected_source = source;
+	if (mgr->priv->selected_source != NULL) {
+		g_object_unref (mgr->priv->selected_source);
+	}
+	mgr->priv->selected_source = g_object_ref (source);
+
 	playlist_active = RB_IS_PLAYLIST_SOURCE (mgr->priv->selected_source);
 	if (playlist_active) {
-		g_object_get (G_OBJECT (mgr->priv->selected_source), "is-local", &playlist_local, NULL);
+		g_object_get (mgr->priv->selected_source, "is-local", &playlist_local, NULL);
 	}
 
 	can_save = playlist_local && !party_mode;
@@ -390,29 +397,30 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 	action = gtk_action_group_get_action (mgr->priv->actiongroup,
 					      "MusicPlaylistRenamePlaylist");
 	gtk_action_set_visible (action, can_rename);
+}
 
-	action = gtk_action_group_get_action (mgr->priv->actiongroup,
-					      "MusicPlaylistBurnPlaylist");
-	if (playlist_active && rb_recorder_enabled ()) {
-		RhythmDBQueryModel *model;
+static void
+rb_playlist_manager_set_shell_internal (RBPlaylistManager *mgr,
+					RBShell           *shell)
+{
+	GtkUIManager *uimanager = NULL;
+	RhythmDB     *db = NULL;
 
-		g_object_get (G_OBJECT (mgr->priv->selected_source), "query-model", &model, NULL);
-		/* monitor for changes, to enable/disable the burn menu item */
-		g_signal_connect_object (G_OBJECT (model),
-					 "row_inserted",
-					 G_CALLBACK (rb_playlist_manager_playlist_row_inserted_cb),
-					 mgr, 0);
-		g_signal_connect_object (G_OBJECT (model),
-					 "post-entry-delete",
-					 G_CALLBACK (rb_playlist_manager_playlist_entries_changed),
-					 mgr, 0);
-
-		rb_playlist_manager_playlist_entries_changed (GTK_TREE_MODEL (model), NULL, mgr);
-		g_object_unref (model);
-		gtk_action_set_visible (action, TRUE);
-	} else {
-		gtk_action_set_visible (action, FALSE);
+	if (mgr->priv->db != NULL) {
+		g_object_unref (mgr->priv->db);
 	}
+
+	mgr->priv->shell = shell;
+
+	if (mgr->priv->shell != NULL) {
+		g_object_get (mgr->priv->shell,
+			      "ui-manager", &uimanager,
+			      "db", &db,
+			      NULL);
+	}
+
+	mgr->priv->db = db;
+	rb_playlist_manager_set_uimanager (mgr, uimanager);
 }
 
 static void
@@ -424,22 +432,19 @@ rb_playlist_manager_set_property (GObject *object,
 	RBPlaylistManager *mgr = RB_PLAYLIST_MANAGER (object);
 
 	switch (prop_id) {
+	case PROP_PLAYLIST_NAME:
+		g_free (mgr->priv->playlists_file);
+		mgr->priv->playlists_file = g_strdup (g_value_get_string (value));
+                break;
 	case PROP_SOURCE:
 		rb_playlist_manager_set_source (mgr, g_value_get_object (value));
 		break;
 	case PROP_SHELL:
-	{
-		GtkUIManager *uimanager;
-		mgr->priv->shell = g_value_get_object (value);
-		g_object_get (G_OBJECT (mgr->priv->shell), 
-			      "ui-manager", &uimanager, 
-			      "db", &mgr->priv->db,
-			      NULL);
-		rb_playlist_manager_set_uimanager (mgr, uimanager);
+		rb_playlist_manager_set_shell_internal (mgr, g_value_get_object (value));
 		break;
-	}
 	case PROP_SOURCELIST:
 		mgr->priv->sourcelist = g_value_get_object (value);
+		g_object_ref (mgr->priv->sourcelist);
 		mgr->priv->window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (mgr->priv->sourcelist)));
 		break;
 	default:
@@ -448,7 +453,7 @@ rb_playlist_manager_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_playlist_manager_get_property (GObject *object,
 			      guint prop_id,
 			      GValue *value,
@@ -457,6 +462,9 @@ rb_playlist_manager_get_property (GObject *object,
 	RBPlaylistManager *mgr = RB_PLAYLIST_MANAGER (object);
 
 	switch (prop_id) {
+        case PROP_PLAYLIST_NAME:
+                g_value_set_string (value, mgr->priv->playlists_file);
+                break;
 	case PROP_SOURCE:
 		g_value_set_object (value, mgr->priv->selected_source);
 		break;
@@ -474,11 +482,13 @@ rb_playlist_manager_get_property (GObject *object,
 
 RBPlaylistManager *
 rb_playlist_manager_new (RBShell *shell,
-			 RBSourceList *sourcelist)
+			 RBSourceList *sourcelist,
+			 const char *playlists_file)
 {
 	return g_object_new (RB_TYPE_PLAYLIST_MANAGER,
 			     "shell", shell,
 			     "sourcelist", sourcelist,
+			     "playlists_file", playlists_file,
 			     NULL);
 }
 
@@ -492,34 +502,47 @@ rb_playlist_manager_error_quark (void)
 	return quark;
 }
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
 static void
-handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri_maybe,
+handle_playlist_entry_cb (TotemPlParser *playlist,
+			  const char *uri_maybe,
+			  GHashTable *metadata,
+			  RBPlaylistManager *mgr)
+#else
+static void
+handle_playlist_entry_cb (TotemPlParser *playlist,
+			  const char *uri_maybe,
 			  const char *title,
-			  const char *genre, RBPlaylistManager *mgr)
+			  const char *genre,
+			  RBPlaylistManager *mgr)
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 {
-	char *uri = rb_canonicalise_uri (uri_maybe);
-	RhythmDBEntryType entry_type;
+	char *uri;
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+	const char *title, *genre;
 
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
+	genre = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_GENRE);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
+
+	uri = rb_canonicalise_uri (uri_maybe);
 	g_return_if_fail (uri != NULL);
 
-	entry_type = rb_shell_guess_type_for_uri (mgr->priv->shell, uri);
-	if (entry_type == RHYTHMDB_ENTRY_TYPE_INVALID) {
-		g_free (uri);
+	rb_debug ("adding uri %s (title %s, genre %s) from playlist",
+		  uri, title, genre);
+	if (!rb_shell_add_uri (mgr->priv->shell,
+			       uri,
+			       title,
+			       genre,
+			       NULL))
 		return;
+
+	if (!mgr->priv->loading_playlist) {
+		mgr->priv->loading_playlist =
+			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
 	}
-
-	rb_shell_add_uri (mgr->priv->shell,
-			  entry_type,
-			  uri,
-			  title,
-			  genre,
-			  NULL);
-
-	if (entry_type == RHYTHMDB_ENTRY_TYPE_SONG) {
-		if (!mgr->priv->loading_playlist) {
-			mgr->priv->loading_playlist =
-				RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, NULL, FALSE));
-		}
+	if (rb_source_want_uri (RB_SOURCE (mgr->priv->loading_playlist), uri) > 0) {
+		rb_debug ("adding uri %s to playlist", uri);
 		rb_static_playlist_source_add_location (mgr->priv->loading_playlist, uri, -1);
 	}
 
@@ -531,17 +554,19 @@ playlist_load_start_cb (TotemPlParser *parser, const char *title, RBPlaylistMana
 {
 	rb_debug ("loading new playlist %s", title);
 
-	if (!mgr->priv->loading_playlist) {
-		mgr->priv->loading_playlist =
+	mgr->priv->loading_playlist =
 			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
-	}
 }
 
 static void
 playlist_load_end_cb (TotemPlParser *parser, const char *title, RBPlaylistManager *mgr)
 {
 	rb_debug ("finished loading playlist %s", title);
-	mgr->priv->loading_playlist = NULL;
+
+	if (title) {
+		g_object_set (mgr->priv->loading_playlist, "name", title, NULL);
+		mgr->priv->loading_playlist = NULL;
+	}
 }
 
 /**
@@ -561,25 +586,31 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 {
 	rb_debug ("loading playlist from %s", uri);
 
-	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_LOAD_START], 0);
+	g_signal_emit (mgr, rb_playlist_manager_signals[PLAYLIST_LOAD_START], 0);
 
 	{
 		TotemPlParser *parser = totem_pl_parser_new ();
 
-		g_signal_connect_object (G_OBJECT (parser), "entry",
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+		g_signal_connect_object (parser, "entry-parsed",
 					 G_CALLBACK (handle_playlist_entry_cb),
 					 mgr, 0);
+#else
+		g_signal_connect_object (parser, "entry",
+					 G_CALLBACK (handle_playlist_entry_cb),
+					 mgr, 0);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
 
-		g_signal_connect_object (G_OBJECT (parser), "playlist-start",
+		g_signal_connect_object (parser, "playlist-start",
 					 G_CALLBACK (playlist_load_start_cb),
 					 mgr, 0);
 
-		g_signal_connect_object (G_OBJECT (parser), "playlist-end",
+		g_signal_connect_object (parser, "playlist-end",
 					 G_CALLBACK (playlist_load_end_cb),
 					 mgr, 0);
-		
+
 		if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
-			g_object_set (G_OBJECT (parser), "recurse", FALSE, NULL);
+			g_object_set (parser, "recurse", FALSE, NULL);
 
 		if (totem_pl_parser_parse (parser, uri, TRUE) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 			g_set_error (error,
@@ -589,19 +620,38 @@ rb_playlist_manager_parse_file (RBPlaylistManager *mgr, const char *uri, GError
 				     _("The playlist file may be in an unknown format or corrupted."));
 			return FALSE;
 		}
-		mgr->priv->loading_playlist = NULL;
+		if (mgr->priv->loading_playlist != NULL) {
+			char *name = NULL;
 
-		g_object_unref (G_OBJECT (parser));
+			/* totem-plparser may not have given us the playlist name */
+			g_object_get (mgr->priv->loading_playlist, "name", &name, NULL);
+			if (name == NULL || name[0] == '\0') {
+				char *path;
+
+				rb_debug ("setting playlist name from file name");
+				path = g_filename_from_uri (uri, NULL, NULL);
+				if (path) {
+					name = g_path_get_basename (path);
+					g_object_set (mgr->priv->loading_playlist, "name", name, NULL);
+					g_free (path);
+				}
+			}
+
+			g_free (name);
+			mgr->priv->loading_playlist = NULL;
+		}
+
+		g_object_unref (parser);
 	}
 
-	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_LOAD_FINISH], 0);
+	g_signal_emit (mgr, rb_playlist_manager_signals[PLAYLIST_LOAD_FINISH], 0);
 	return TRUE;
 }
 
 static void
 append_new_playlist_source (RBPlaylistManager *mgr, RBPlaylistSource *source)
 {
-	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_ADDED], 0,
+	g_signal_emit (mgr, rb_playlist_manager_signals[PLAYLIST_ADDED], 0,
 		       source);
 }
 
@@ -620,9 +670,9 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 	xmlNodePtr root;
 	xmlNodePtr child;
 	gboolean exists;
-	
+
 	exists = FALSE;
-	file = g_build_filename (rb_dot_dir (), "playlists.xml", NULL);
+	file = g_strdup (mgr->priv->playlists_file);
 
 	/* block saves until the playlists have loaded */
 	g_mutex_lock (mgr->priv->saving_mutex);
@@ -647,7 +697,7 @@ rb_playlist_manager_load_playlists (RBPlaylistManager *mgr)
 		goto out;
 
 	root = xmlDocGetRootElement (doc);
-	
+
 	for (child = root->children; child; child = child->next) {
 		RBSource *playlist;
 
@@ -681,7 +731,7 @@ rb_playlist_manager_is_dirty (RBPlaylistManager *mgr)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
-	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
@@ -689,25 +739,29 @@ rb_playlist_manager_is_dirty (RBPlaylistManager *mgr)
 		do {
 			RBSource *source;
 			gboolean local;
-			GValue v = {0,};
-			gtk_tree_model_get_value (model,
-						  &iter,
-						  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-						  &v);
-			source = g_value_get_pointer (&v);
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
-				continue;
 
-			g_object_get (G_OBJECT (source), 
+			gtk_tree_model_get (model,
+					    &iter,
+					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+					    &source,
+					    -1);
+			if (source == NULL) {
+				continue;
+			}
+			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
+				g_object_unref (source);
+				continue;
+			}
+
+			g_object_get (source,
 				      "is-local", &local,
 				      NULL);
 			if (local) {
-				g_object_get (G_OBJECT (source),
-					      "dirty", &dirty,
-					      NULL);
+				g_object_get (source, "dirty", &dirty, NULL);
 				if (dirty)
 					break;
 			}
+			g_object_unref (source);
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 
@@ -731,9 +785,9 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 
 	g_mutex_lock (data->mgr->priv->saving_mutex);
 
-	file = g_build_filename (rb_dot_dir (), "playlists.xml", NULL);
+	file = g_strdup (data->mgr->priv->playlists_file);
 	tmpname = g_strconcat (file, ".tmp", NULL);
-	
+
 	if (xmlSaveFormatFile (tmpname, data->doc, 1) != -1) {
 		rename (tmpname, file);
 	} else {
@@ -748,7 +802,7 @@ rb_playlist_manager_save_data (struct RBPlaylistManagerSaveData *data)
 	g_atomic_int_compare_and_exchange (&data->mgr->priv->saving, 1, 0);
 	g_mutex_unlock (data->mgr->priv->saving_mutex);
 
-	g_object_unref (G_OBJECT (data->mgr));
+	g_object_unref (data->mgr);
 
 	g_free (data);
 	return NULL;
@@ -785,36 +839,42 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
 		/* already saving, so don't bother */
 		return FALSE;
 	}
-	
+
 	data = g_new0 (struct RBPlaylistManagerSaveData, 1);
 	data->mgr = mgr;
 	data->doc = xmlNewDoc (RB_PLAYLIST_MGR_VERSION);
-	g_object_ref (G_OBJECT (mgr));
+	g_object_ref (mgr);
 
 	root = xmlNewDocNode (data->doc, NULL, RB_PLAYLIST_MGR_PL, NULL);
 	xmlDocSetRootElement (data->doc, root);
-	
-	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+
+	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
 
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		do {
 			RBSource *source;
-			GValue v = {0,};
 			gboolean local;
 
-			gtk_tree_model_get_value (model,
-						  &iter,
-						  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-						  &v);
-			source = g_value_get_pointer (&v);
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
+			gtk_tree_model_get (model,
+					    &iter,
+					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+					    &source,
+					    -1);
+			if (source == NULL) {
 				continue;
+			}
+			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
+				g_object_unref (source);
+				continue;
+			}
 
-			g_object_get (G_OBJECT (source), "is-local", &local, NULL);
+			g_object_get (source, "is-local", &local, NULL);
 			if (local)
 				rb_playlist_source_save_to_xml (RB_PLAYLIST_SOURCE (source), root);
+
+			g_object_unref (source);
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
 
@@ -841,25 +901,25 @@ rb_playlist_manager_save_playlists (RBPlaylistManager *mgr, gboolean force)
  **/
 RBSource *
 rb_playlist_manager_new_playlist (RBPlaylistManager *mgr,
-				  const char *suggested_name, 
+				  const char *suggested_name,
 				  gboolean automatic)
 {
 	RBSource *playlist;
 	if (automatic)
-		playlist = rb_auto_playlist_source_new (mgr->priv->shell, 
-							suggested_name, 
+		playlist = rb_auto_playlist_source_new (mgr->priv->shell,
+							suggested_name,
 							TRUE);
 	else
-		playlist = rb_static_playlist_source_new (mgr->priv->shell, 
-							  suggested_name, 
-							  TRUE, 
+		playlist = rb_static_playlist_source_new (mgr->priv->shell,
+							  suggested_name,
+							  TRUE,
 							  RHYTHMDB_ENTRY_TYPE_SONG);
-	
+
 	append_new_playlist_source (mgr, RB_PLAYLIST_SOURCE (playlist));
 	rb_sourcelist_edit_source_name (mgr->priv->sourcelist, playlist);
 	rb_playlist_manager_set_dirty (mgr, TRUE);
-	
-	g_signal_emit (G_OBJECT (mgr), rb_playlist_manager_signals[PLAYLIST_CREATED], 0,
+
+	g_signal_emit (mgr, rb_playlist_manager_signals[PLAYLIST_CREATED], 0,
 		       playlist);
 
 	return playlist;
@@ -872,8 +932,11 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
 	char  *name = NULL;
 	GList *list;
 
-        if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
-                list = gnome_vfs_uri_list_parse ((char *) data->data);
+        if (data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE)) {
+		gboolean is_id;
+		list = rb_uri_list_parse ((const char *)data->data);
+		is_id = (data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE));
 
 		if (list != NULL) {
 			GList   *l;
@@ -888,16 +951,12 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
 			mixed_albums  = FALSE;
 			for (l = list; l != NULL; l = g_list_next (l)) {
 				RhythmDBEntry *entry;
-				char          *location;
 				const char    *e_artist;
 				const char    *e_album;
 
-				location = gnome_vfs_uri_to_string ((const GnomeVFSURI *) l->data, 0);
-				if (location == NULL) {
-					continue;
-				}
-
-				entry = rhythmdb_entry_lookup_by_location (mgr->priv->db, location);
+				entry = rhythmdb_entry_lookup_from_string (mgr->priv->db,
+									   (const char *)l->data,
+									   is_id);
 				if (entry == NULL) {
 					continue;
 				}
@@ -943,7 +1002,7 @@ create_name_from_selection_data (RBPlaylistManager *mgr,
 
 			g_free (artist);
 			g_free (album);
-			gnome_vfs_uri_list_free (list);
+			rb_list_deep_free (list);
 		}
 
 	} else {
@@ -977,14 +1036,16 @@ rb_playlist_manager_new_playlist_from_selection_data (RBPlaylistManager *mgr,
 						      GtkSelectionData *data)
 {
 	RBSource *playlist;
-	gboolean  automatic;
+	gboolean  automatic = TRUE;
 	char     *suggested_name;
 
-	automatic = (data->type != gdk_atom_intern ("text/uri-list", TRUE));
+	if (data->type == gdk_atom_intern ("text/uri-list", TRUE) ||
+	    data->type == gdk_atom_intern ("application/x-rhythmbox-entry", TRUE))
+		automatic = FALSE;
 	suggested_name = create_name_from_selection_data (mgr, data);
 
 	playlist = rb_playlist_manager_new_playlist (mgr,
-						     suggested_name, 
+						     suggested_name,
 						     automatic);
 	g_free (suggested_name);
 
@@ -1007,14 +1068,21 @@ rb_playlist_manager_set_automatic_playlist (RBPlaylistManager *mgr,
 	GValueArray *limit_value = NULL;
 	const char *sort_key;
 	gint sort_direction;
+	GPtrArray *query;
 
 	rb_query_creator_get_limit (creator, &limit_type, &limit_value);
-	rb_query_creator_get_sort_order (creator, &sort_key, &sort_direction);
+	rb_query_creator_get_sort_order (creator,
+					 &sort_key,
+					 &sort_direction);
 
+	query = rb_query_creator_get_query (creator);
 	rb_auto_playlist_source_set_query (RB_AUTO_PLAYLIST_SOURCE (playlist),
-					   rb_query_creator_get_query (creator),
-					   limit_type, limit_value,
-					   sort_key, sort_direction);
+					   query,
+					   limit_type,
+					   limit_value,
+					   sort_key,
+					   sort_direction);
+	rhythmdb_query_free (query);
 }
 
 static void
@@ -1023,21 +1091,21 @@ rb_playlist_manager_cmd_new_automatic_playlist (GtkAction *action,
 {
 	RBQueryCreator *creator = RB_QUERY_CREATOR (rb_query_creator_new (mgr->priv->db));
 	RBSource *playlist;
-	
+
 	switch (gtk_dialog_run (GTK_DIALOG (creator))) {
 	case GTK_RESPONSE_NONE:
 	case GTK_RESPONSE_CLOSE:
-		gtk_widget_destroy (GTK_WIDGET (creator));	
+		gtk_widget_destroy (GTK_WIDGET (creator));
 		return;
 	}
 
 	playlist = rb_playlist_manager_new_playlist (mgr, NULL, TRUE);
 
-	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator); 
+	rb_playlist_manager_set_automatic_playlist (mgr, RB_AUTO_PLAYLIST_SOURCE (playlist), creator);
 
 	rb_playlist_manager_set_dirty (mgr, TRUE);
 
-	gtk_widget_destroy (GTK_WIDGET (creator));	
+	gtk_widget_destroy (GTK_WIDGET (creator));
 }
 
 typedef struct {
@@ -1089,35 +1157,43 @@ rb_playlist_manager_cmd_edit_automatic_playlist (GtkAction *action,
 		RhythmDBQueryModelLimitType limit_type;
 		GValueArray *limit_value = NULL;
 		GPtrArray *query;
-		const char *sort_key;
+		char *sort_key;
 		gint sort_direction;
 		EditAutoPlaylistData *data;
 
-		rb_auto_playlist_source_get_query (playlist, &query,
-						   &limit_type, &limit_value,
-						   &sort_key, &sort_direction);
+		sort_key = NULL;
+		rb_auto_playlist_source_get_query (playlist,
+						   &query,
+						   &limit_type,
+						   &limit_value,
+						   &sort_key,
+						   &sort_direction);
 
 		creator = RB_QUERY_CREATOR (rb_query_creator_new_from_query (mgr->priv->db,
 									     query,
-									     limit_type, limit_value,
-									     sort_key, sort_direction));
-		if (limit_value)
+									     limit_type,
+									     limit_value,
+									     sort_key,
+									     sort_direction));
+		if (limit_value != NULL) {
 			g_value_array_free (limit_value);
+		}
 		rhythmdb_query_free (query);
+		g_free (sort_key);
 
 		data = g_new0 (EditAutoPlaylistData, 1);
 		data->mgr = mgr;
 		data->playlist = playlist;
 		data->creator = creator;
-		data->creator_response_id = 
-			g_signal_connect (G_OBJECT (creator),
+		data->creator_response_id =
+			g_signal_connect (creator,
 					  "response",
 					  G_CALLBACK (edit_auto_playlist_response_cb),
 					  data);
-		
+
 		g_object_set_data (G_OBJECT (playlist), "rhythmbox-playlist-editor", creator);
-		data->playlist_deleted_id = 
-			g_signal_connect (G_OBJECT (playlist),
+		data->playlist_deleted_id =
+			g_signal_connect (playlist,
 					  "deleted",
 					  G_CALLBACK (edit_auto_playlist_deleted_cb),
 					  data);
@@ -1132,10 +1208,11 @@ _queue_track_cb (RhythmDBQueryModel *model,
 		 RBStaticPlaylistSource *queue_source)
 {
 	RhythmDBEntry *entry;
-	
+
 	entry = rhythmdb_query_model_iter_to_entry (model, iter);
 	rb_static_playlist_source_add_entry (queue_source, entry, -1);
-	
+	rhythmdb_entry_unref (entry);
+
 	return FALSE;
 }
 
@@ -1146,15 +1223,15 @@ rb_playlist_manager_cmd_queue_playlist (GtkAction *action,
 	RBSource *queue_source;
 	RhythmDBQueryModel *model;
 
-	g_object_get (G_OBJECT (mgr->priv->shell), "queue-source", &queue_source, NULL);
-	g_object_get (G_OBJECT (mgr->priv->selected_source), "query-model", &model, NULL);
+	g_object_get (mgr->priv->shell, "queue-source", &queue_source, NULL);
+	g_object_get (mgr->priv->selected_source, "query-model", &model, NULL);
 
 	gtk_tree_model_foreach (GTK_TREE_MODEL (model),
 				(GtkTreeModelForeachFunc) _queue_track_cb,
 				queue_source);
 
-	g_object_unref (G_OBJECT (queue_source));
-	g_object_unref (G_OBJECT (model));
+	g_object_unref (queue_source);
+	g_object_unref (model);
 }
 
 static void
@@ -1172,7 +1249,7 @@ rb_playlist_manager_cmd_delete_playlist (GtkAction *action,
 					 RBPlaylistManager *mgr)
 {
 	rb_debug ("Deleting playlist %p", mgr->priv->selected_source);
-	
+
 	rb_source_delete_thyself (mgr->priv->selected_source);
 	rb_playlist_manager_set_dirty (mgr, TRUE);
 }
@@ -1218,12 +1295,10 @@ rb_playlist_manager_cmd_load_playlist (GtkAction *action,
 				      GTK_FILE_CHOOSER_ACTION_OPEN,
 				      FALSE);
 
-	g_signal_connect_object (G_OBJECT (dialog), "response",
+	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (load_playlist_response_cb), mgr, 0);
 }
 
-
-
 typedef struct {
   const gchar *description;
   /* NULL terminated array of extensions for this file format.  The first
@@ -1284,7 +1359,7 @@ save_playlist_response_cb (GtkDialog *dialog,
 	if (export_type == RB_PLAYLIST_EXPORT_TYPE_UNKNOWN) {
 			rb_error_dialog (NULL, _("Couldn't save playlist"), _("Unsupported file extension given."));
 	} else {
-		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source), 
+		rb_playlist_source_save_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source),
 						  file, (export_type == RB_PLAYLIST_EXPORT_TYPE_M3U));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	}
@@ -1301,7 +1376,7 @@ export_set_extension_cb (GtkWidget* widget, GtkDialog *dialog)
 	const char *extension;
 	gchar *basename;
 	GString *basename_str;
-  
+
 	index = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 	if (index <= 0)
 		return;
@@ -1309,7 +1384,7 @@ export_set_extension_cb (GtkWidget* widget, GtkDialog *dialog)
 	extension = playlist_export_formats[index-1].extensions[0];
 	if (extension == NULL)
 		return;
-	
+
 	text = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 	if (text == NULL || text[0] == '\0') {
 		g_free (text);
@@ -1323,7 +1398,7 @@ export_set_extension_cb (GtkWidget* widget, GtkDialog *dialog)
 		g_string_truncate (basename_str, (last_dot-basename));
 	g_free (basename);
 	g_free (text);
-  
+
 	g_string_append_printf (basename_str, ".%s", extension);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), basename_str->str);
 	g_string_free (basename_str, TRUE);
@@ -1343,10 +1418,10 @@ filter_get_export_filter_label (RBPlaylistExportFilter *efilter)
 			g_string_append (str, ", *.");
 		g_string_append (str, efilter->extensions[ext]);
 	}
-    
+
 	if (ext > 0)
 		g_string_append (str, ")");
-    
+
 	return g_string_free (str, FALSE);
 }
 
@@ -1363,15 +1438,15 @@ setup_format_menu (GtkWidget* menu, GtkWidget *dialog)
 	for (i = 0; i < G_N_ELEMENTS (playlist_export_formats); i++) {
 		gchar *filter_label;
 		GtkTreeIter iter;
-    
+
 		filter_label = filter_get_export_filter_label (&playlist_export_formats[i]);
 		gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, -1,
 						   0, filter_label, -1);
 
 		g_free (filter_label);
 	}
-    
-	g_signal_connect_object (GTK_OBJECT (menu),
+
+	g_signal_connect_object (menu,
 				 "changed", G_CALLBACK (export_set_extension_cb),
 				 dialog, 0);
 }
@@ -1382,6 +1457,7 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 {
 	GladeXML *xml;
 	GtkWidget *dialog, *menu;
+	char *name, *tmp;
 
 	xml = rb_glade_xml_new ("playlist-save.glade",
 				"playlist_save_dialog",
@@ -1391,21 +1467,20 @@ rb_playlist_manager_cmd_save_playlist (GtkAction *action,
 	menu = glade_xml_get_widget (xml, "playlist_format_menu");
 	setup_format_menu (menu, dialog);
 	g_object_set_data (G_OBJECT (dialog), "export-menu", menu);
-	
+
+	g_object_get (mgr->priv->selected_source, "name", &name, NULL);
+	tmp = g_strconcat (name, ".pls", NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), tmp);
+	g_free (tmp);
+	g_free (name);
+
 	/* FIXME: always has "by extension" as default (it should probably remember the last selection) */
 	gtk_combo_box_set_active (GTK_COMBO_BOX (menu), 0);
-	g_signal_connect_object (G_OBJECT (dialog), "response",
+	g_signal_connect_object (dialog, "response",
 				 G_CALLBACK (save_playlist_response_cb),
 				 mgr, 0);
 
-	g_object_unref (G_OBJECT (xml));
-}
-
-static void
-rb_playlist_manager_cmd_burn_playlist (GtkAction *action,
-				       RBPlaylistManager *mgr)
-{
-	rb_playlist_source_burn_playlist (RB_PLAYLIST_SOURCE (mgr->priv->selected_source));
+	g_object_unref (xml);
 }
 
 /**
@@ -1421,35 +1496,40 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
 	GtkTreeIter iter;
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
-	
-	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+
+	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
-	
+
 	if (gtk_tree_model_get_iter_first (model, &iter)) {
 		do {
 			RBSource *source;
-			GValue v = {0,};
 			gboolean local;
 
-			gtk_tree_model_get_value (model,
-						  &iter,
-						  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-						  &v);
-			source = g_value_get_pointer (&v);
-			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE)
+			gtk_tree_model_get (model,
+					    &iter,
+					    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+					    &source,
+					    -1);
+			if (source == NULL) {
 				continue;
-			if (RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE)
+			}
+			if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
+			    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
+				g_object_unref (source);
 				continue;
-			g_object_get (G_OBJECT (source), "is-local", &local, 
+			}
+
+			g_object_get (source, "is-local", &local,
 				      NULL);
 			if (local) {
 				playlists = g_list_prepend (playlists, source);
 			}
 
+			g_object_unref (source);
 		} while (gtk_tree_model_iter_next (model, &iter));
 	}
-	
+
 	return playlists;
 }
 
@@ -1461,7 +1541,7 @@ rb_playlist_manager_get_playlists (RBPlaylistManager *mgr)
  *
  * Allocates and returns an array containing the names of all local
  * playlists.  This is part of the playlist manager dbus interface.
- * 
+ *
  * Returns TRUE if successful.
  **/
 gboolean
@@ -1473,11 +1553,11 @@ rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
 	GtkTreeModel *fmodel;
 	GtkTreeModel *model;
 	int i;
-	
-	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+
+	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
 	g_object_unref (fmodel);
-	
+
 	if (!gtk_tree_model_get_iter_first (model, &iter)) {
 		*playlists = NULL;
 		return TRUE;
@@ -1490,23 +1570,29 @@ rb_playlist_manager_get_playlist_names (RBPlaylistManager *mgr,
 	i = 0;
 	do {
 		RBSource *source;
-		GValue v = {0,};
 		char *source_name;
 
-		gtk_tree_model_get_value (model, &iter,
-					  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					  &v);
-		source = g_value_get_pointer (&v);
-		if (!RB_IS_PLAYLIST_SOURCE (source))
+		gtk_tree_model_get (model, &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+				    &source,
+				    -1);
+		if (source == NULL) {
 			continue;
-		if (RB_IS_PLAY_QUEUE_SOURCE (source))
+		}
+		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE
+		    || RB_IS_PLAY_QUEUE_SOURCE (source) == TRUE) {
+			g_object_unref (source);
 			continue;
+		}
 
-		g_object_get (G_OBJECT (source), "name", &source_name, NULL);
+		g_object_get (source, "name", &source_name, NULL);
 		(*playlists)[i++] = source_name;
+
+		g_object_unref (source);
+
 	} while (gtk_tree_model_iter_next (model, &iter));
 
-	return TRUE;	
+	return TRUE;
 }
 
 static RBSource *
@@ -1518,30 +1604,34 @@ _get_playlist_by_name (RBPlaylistManager *mgr,
 	GtkTreeModel *model;
 	RBSource *playlist = NULL;
 
-	g_object_get (G_OBJECT (mgr->priv->sourcelist), "model", &fmodel, NULL);
+	g_object_get (mgr->priv->sourcelist, "model", &fmodel, NULL);
 	model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (fmodel));
-	g_object_unref (G_OBJECT (fmodel));
+	g_object_unref (fmodel);
 
 	if (!gtk_tree_model_get_iter_first (model, &iter))
 		return NULL;
 
 	do {
 		RBSource *source;
-		GValue v = {0,};
 		char *source_name;
-		
-		gtk_tree_model_get_value (model, &iter,
-					  RB_SOURCELIST_MODEL_COLUMN_SOURCE,
-					  &v);
-		source = g_value_get_pointer (&v);
-		if (!RB_IS_PLAYLIST_SOURCE (source))
+
+		gtk_tree_model_get (model, &iter,
+				    RB_SOURCELIST_MODEL_COLUMN_SOURCE,
+				    &source,
+				    -1);
+		if (source == NULL) {
 			continue;
-		g_object_get (G_OBJECT (source), "name", &source_name, NULL);
+		}
+		if (RB_IS_PLAYLIST_SOURCE (source) == FALSE) {
+			g_object_unref (source);
+			continue;
+		}
+		g_object_get (source, "name", &source_name, NULL);
 		if (strcmp (name, source_name) == 0)
 			playlist = source;
 
 		g_free (source_name);
-
+		g_object_unref (source);
 	} while (gtk_tree_model_iter_next (model, &iter) && playlist == NULL);
 
 	return playlist;
@@ -1556,7 +1646,7 @@ _get_playlist_by_name (RBPlaylistManager *mgr,
  * Creates a new static playlist source with the given name.
  * Will fail if a playlist with that name already exists.
  * This is part of the playlist manager dbus interface.
- * 
+ *
  * Returns TRUE if successful.
  **/
 gboolean
@@ -1589,8 +1679,8 @@ rb_playlist_manager_create_static_playlist (RBPlaylistManager *mgr,
  * Returns TRUE if successful.
  */
 gboolean
-rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr, 
-				     const gchar *name, 
+rb_playlist_manager_delete_playlist (RBPlaylistManager *mgr,
+				     const gchar *name,
 				     GError **error)
 {
 	RBSource *playlist = _get_playlist_by_name (mgr, name);
@@ -1724,4 +1814,3 @@ rb_playlist_manager_export_playlist (RBPlaylistManager *mgr,
 					  m3u_format);
 	return TRUE;
 }
-
