@@ -35,12 +35,6 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
-#ifdef HAVE_MMKEYS
-#include <X11/Xlib.h>
-#include <X11/XF86keysym.h>
-#include <gdk/gdkx.h>
-#endif /* HAVE_MMKEYS */
-
 #include "rb-property-view.h"
 #include "rb-shell-player.h"
 #include "rb-stock-icons.h"
@@ -77,6 +71,7 @@ static void rb_shell_player_class_init (RBShellPlayerClass *klass);
 static void rb_shell_player_init (RBShellPlayer *shell_player);
 static GObject *rb_shell_player_constructor (GType type, guint n_construct_properties,
 					     GObjectConstructParam *construct_properties);
+static void rb_shell_player_dispose (GObject *object);
 static void rb_shell_player_finalize (GObject *object);
 static void rb_shell_player_set_property (GObject *object,
 					  guint prop_id,
@@ -154,14 +149,6 @@ static void rb_shell_player_extra_metadata_cb (RhythmDB *db,
 
 static gboolean rb_shell_player_jump_to_current_idle (RBShellPlayer *player);
 
-#ifdef HAVE_MMKEYS
-static void grab_mmkey (int key_code, GdkWindow *root);
-static GdkFilterReturn filter_mmkeys (GdkXEvent *xevent,
-				      GdkEvent *event,
-				      gpointer data);
-static void rb_shell_player_init_mmkeys (RBShellPlayer *shell_player);
-#endif /* HAVE_MMKEYS */
-
 #define CONF_STATE		CONF_PREFIX "/state"
 
 struct RBShellPlayerPrivate
@@ -281,6 +268,7 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = rb_shell_player_dispose;
 	object_class->finalize = rb_shell_player_finalize;
 	object_class->constructor = rb_shell_player_constructor;
 
@@ -735,11 +723,6 @@ rb_shell_player_init (RBShellPlayer *player)
 		eel_gconf_notification_add (CONF_UI_SONG_POSITION_SLIDER_HIDDEN,
 					    (GConfClientNotifyFunc) gconf_song_position_slider_visibility_changed,
 					    player);
-
-#ifdef HAVE_MMKEYS
-	/* Enable Multimedia Keys */
-	rb_shell_player_init_mmkeys (player);
-#endif /* HAVE_MMKEYS */
 }
 
 static void
@@ -749,6 +732,7 @@ rb_shell_player_set_source_internal (RBShellPlayer *player,
 	if (player->priv->selected_source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (player->priv->selected_source);
 		GList *property_views = rb_source_get_property_views (player->priv->selected_source);
+		GList *l;
 
 		if (songs != NULL) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
@@ -756,8 +740,8 @@ rb_shell_player_set_source_internal (RBShellPlayer *player,
 							      player);
 		}
 
-		for (; property_views; property_views = property_views->next) {
-			g_signal_handlers_disconnect_by_func (G_OBJECT (property_views->data),
+		for (l = property_views; l != NULL; l = g_list_next (l)) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (l->data),
 							      G_CALLBACK (rb_shell_player_property_row_activated_cb),
 							      player);
 		}
@@ -775,17 +759,19 @@ rb_shell_player_set_source_internal (RBShellPlayer *player,
 	if (player->priv->selected_source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (player->priv->selected_source);
 		GList *property_views = rb_source_get_property_views (player->priv->selected_source);
+		GList *l;
 
 		if (songs)
 			g_signal_connect_object (G_OBJECT (songs),
 						 "entry-activated",
 						 G_CALLBACK (rb_shell_player_entry_activated_cb),
 						 player, 0);
-		for (; property_views; property_views = property_views->next)
-			g_signal_connect_object (G_OBJECT (property_views->data),
+		for (l = property_views; l != NULL; l = g_list_next (l)) {
+			g_signal_connect_object (G_OBJECT (l->data),
 						 "property-activated",
 						 G_CALLBACK (rb_shell_player_property_row_activated_cb),
 						 player, 0);
+		}
 
 		g_list_free (property_views);
 	}
@@ -878,6 +864,41 @@ rb_shell_player_set_queue_source_internal (RBShellPlayer     *player,
 	}
 }
 
+static void
+rb_shell_player_dispose (GObject *object)
+{
+	RBShellPlayer *player;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RB_IS_SHELL_PLAYER (object));
+
+	player = RB_SHELL_PLAYER (object);
+
+	g_return_if_fail (player->priv != NULL);
+
+	if (player->priv->gconf_play_order_id != 0) {
+		eel_gconf_notification_remove (player->priv->gconf_play_order_id);
+		player->priv->gconf_play_order_id = 0;
+	}
+
+	if (player->priv->mmplayer != NULL) {
+		g_object_unref (player->priv->mmplayer);
+		player->priv->mmplayer = NULL;
+	}
+
+	if (player->priv->play_order != NULL) {
+		g_object_unref (player->priv->play_order);
+		player->priv->play_order = NULL;
+	}
+
+	if (player->priv->queue_play_order != NULL) {
+		g_object_unref (player->priv->queue_play_order);
+		player->priv->queue_play_order = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_shell_player_parent_class)->dispose (object);
+}
+
 static void
 rb_shell_player_finalize (GObject *object)
 {
@@ -890,14 +911,8 @@ rb_shell_player_finalize (GObject *object)
 
 	g_return_if_fail (player->priv != NULL);
 
-	eel_gconf_notification_remove (player->priv->gconf_play_order_id);
-
 	eel_gconf_set_float (CONF_STATE_VOLUME, player->priv->volume);
 
-	g_object_unref (player->priv->mmplayer);
-	g_object_unref (player->priv->play_order);
-	g_object_unref (player->priv->queue_play_order);
-
 	G_OBJECT_CLASS (rb_shell_player_parent_class)->finalize (object);
 }
 
@@ -975,7 +990,10 @@ rb_shell_player_get_property (GObject *object,
 		break;
 	}
 	case PROP_PLAYING:
-		g_value_set_boolean (value, rb_player_playing (player->priv->mmplayer));
+		if (player->priv->mmplayer != NULL)
+			g_value_set_boolean (value, rb_player_playing (player->priv->mmplayer));
+		else
+			g_value_set_boolean (value, FALSE);
 		break;
 	case PROP_VOLUME:
 		g_value_set_float (value, player->priv->volume);
@@ -1075,12 +1093,20 @@ typedef struct {
 	char *location;
 } OpenLocationThreadData;
 
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+static void
+playlist_entry_cb (TotemPlParser *playlist,
+		   const char *uri,
+		   GHashTable *metadata,
+		   RBShellPlayer *player)
+#else
 static void
 playlist_entry_cb (TotemPlParser *playlist,
 		   const char *uri,
 		   const char *title,
 		   const char *genre,
 		   RBShellPlayer *player)
+#endif
 {
 	rb_debug ("adding stream url %s", uri);
 	g_queue_push_tail (player->priv->playlist_urls, g_strdup (uri));
@@ -1093,9 +1119,17 @@ open_location_thread (OpenLocationThreadData *data)
 	TotemPlParserResult playlist_result;
 
 	playlist = totem_pl_parser_new ();
+
+#if TOTEM_PL_PARSER_CHECK_VERSION(2,19,0)
+	g_signal_connect_data (G_OBJECT (playlist), "entry-parsed",
+			       G_CALLBACK (playlist_entry_cb),
+			       data->player, NULL, 0);
+#else
 	g_signal_connect_data (G_OBJECT (playlist), "entry",
 			       G_CALLBACK (playlist_entry_cb),
 			       data->player, NULL, 0);
+#endif /* TOTEM_PL_PARSER_CHECK_VERSION */
+
 	totem_pl_parser_add_ignored_mimetype (playlist, "x-directory/normal");
 
 	playlist_result = totem_pl_parser_parse (playlist, data->location, FALSE);
@@ -2193,7 +2227,7 @@ rb_shell_player_property_row_activated_cb (RBPropertyView *view,
 	RhythmDBEntry *entry = NULL;
 	GError *error = NULL;
 
-	rb_debug  ("got property activated");
+	rb_debug ("got property activated");
 
 	rb_shell_player_set_playing_source (player, player->priv->selected_source);
 
@@ -2844,124 +2878,6 @@ rb_shell_player_get_playing_path (RBShellPlayer *shell_player,
 	return TRUE;
 }
 
-#ifdef HAVE_MMKEYS
-static void
-grab_mmkey (int key_code,
-	    GdkWindow *root)
-{
-	gdk_error_trap_push ();
-
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  0,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod2Mask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod5Mask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  LockMask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod2Mask | Mod5Mask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod2Mask | LockMask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod5Mask | LockMask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-	XGrabKey (GDK_DISPLAY (), key_code,
-		  Mod2Mask | Mod5Mask | LockMask,
-		  GDK_WINDOW_XID (root), True,
-		  GrabModeAsync, GrabModeAsync);
-
-	gdk_flush ();
-        if (gdk_error_trap_pop ()) {
-		rb_debug ("Error grabbing key");
-	}
-}
-
-static GdkFilterReturn
-filter_mmkeys (GdkXEvent *xevent,
-	       GdkEvent *event,
-	       gpointer data)
-{
-	XEvent *xev;
-	XKeyEvent *key;
-	RBShellPlayer *player;
-	xev = (XEvent *) xevent;
-	if (xev->type != KeyPress) {
-		return GDK_FILTER_CONTINUE;
-	}
-
-	key = (XKeyEvent *) xevent;
-
-	player = (RBShellPlayer *)data;
-
-	if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay) == key->keycode) {
-		rb_shell_player_playpause (player, FALSE, NULL);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause) == key->keycode) {
-		rb_shell_player_pause (player, NULL);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop) == key->keycode) {
-		rb_shell_player_stop (player);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev) == key->keycode) {
-		rb_shell_player_cmd_previous (NULL, player);
-		return GDK_FILTER_REMOVE;
-	} else if (XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext) == key->keycode) {
-		rb_shell_player_cmd_next (NULL, player);
-		return GDK_FILTER_REMOVE;
-	} else {
-		return GDK_FILTER_CONTINUE;
-	}
-}
-
-static void
-rb_shell_player_init_mmkeys (RBShellPlayer *shell_player)
-{
-	gint keycodes[] = {0, 0, 0, 0, 0};
-	GdkDisplay *display;
-	GdkScreen *screen;
-	GdkWindow *root;
-	guint i, j;
-
-	keycodes[0] = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay);
-	keycodes[1] = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop);
-	keycodes[2] = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev);
-	keycodes[3] = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext);
-	keycodes[4] = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPause);
-
-	display = gdk_display_get_default ();
-
-	for (i = 0; i < gdk_display_get_n_screens (display); i++) {
-		screen = gdk_display_get_screen (display, i);
-
-		if (screen != NULL) {
-			root = gdk_screen_get_root_window (screen);
-
-			for (j = 0; j < G_N_ELEMENTS (keycodes) ; j++) {
-				if (keycodes[j] != 0)
-					grab_mmkey (keycodes[j], root);
-			}
-
-			gdk_window_add_filter (root, filter_mmkeys,
-					       (gpointer) shell_player);
-		}
-	}
-}
-#endif /* HAVE_MMKEYS */
-
 static gboolean
 _idle_unblock_signal_cb (gpointer data)
 {
