@@ -45,7 +45,7 @@
 #include "totem-pl-parser.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
-#include "rb-new-podcast-dialog.h"
+#include "rb-uri-dialog.h"
 #include "rb-podcast-properties-dialog.h"
 #include "rb-feed-podcast-properties-dialog.h"
 #include "rb-playlist-manager.h"
@@ -433,8 +433,26 @@ rb_podcast_source_dispose (GObject *object)
 
 	if (source->priv->db != NULL) {
 		g_object_unref (source->priv->db);
+		source->priv->db = NULL;
 	}
 
+	if (source->priv->cached_all_query) {
+		g_object_unref (source->priv->cached_all_query);
+		source->priv->cached_all_query = NULL;
+	}
+
+	if (source->priv->action_group != NULL) {
+		g_object_unref (source->priv->action_group);
+		source->priv->action_group = NULL;
+	}
+
+	if (source->priv->podcast_mgr != NULL) {
+		g_object_unref (source->priv->podcast_mgr);
+		source->priv->podcast_mgr = NULL;
+	}
+
+	eel_gconf_notification_remove (source->priv->prefs_notify_id);
+
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->dispose (object);
 }
 
@@ -452,22 +470,11 @@ rb_podcast_source_finalize (GObject *object)
 
 	rb_debug ("finalizing podcast source");
 
-	if (source->priv->action_group != NULL) {
-		g_object_unref (source->priv->action_group);
-	}
-
-	eel_gconf_notification_remove (source->priv->prefs_notify_id);
-
-	g_object_unref (source->priv->podcast_mgr);
-
 	if (source->priv->selected_feeds) {
 		g_list_foreach (source->priv->selected_feeds, (GFunc) g_free, NULL);
 	        g_list_free (source->priv->selected_feeds);
 	}
 
-       if (source->priv->cached_all_query)
-		g_object_unref (G_OBJECT (source->priv->cached_all_query));
-
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->finalize (object);
 }
 
@@ -1633,11 +1640,7 @@ rb_podcast_source_post_date_cell_data_func (GtkTreeViewColumn *column,
         if (value == 0) {
 		str = g_strdup (_("Unknown"));
 	} else {
-		struct tm time_tm;
-		time_t time = (time_t) value;
-
-		localtime_r (&time, &time_tm);
-		str = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), &time_tm);
+		str = rb_utf_friendly_time (value);
 	}
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
@@ -1971,6 +1974,14 @@ impl_get_search_actions (RBSource *source)
 	return actions;
 }
 
+static void
+rb_podcast_source_location_added_cb (RBURIDialog *dialog,
+				     const char *location,
+				     RBPodcastSource *source)
+{
+	rb_podcast_manager_subscribe_feed (source->priv->podcast_mgr, location);
+}
+
 static void
 rb_podcast_source_cmd_new_podcast (GtkAction *action,
 				   RBPodcastSource *source)
@@ -1979,7 +1990,11 @@ rb_podcast_source_cmd_new_podcast (GtkAction *action,
 
 	rb_debug ("Got new podcast command");
 
-	dialog = rb_new_podcast_dialog_new (source->priv->podcast_mgr);
+	dialog = rb_uri_dialog_new (_("New Podcast Feed"), _("URL of podcast feed:"));
+	g_signal_connect_object (G_OBJECT (dialog),
+				 "location-added",
+				 G_CALLBACK (rb_podcast_source_location_added_cb),
+				 source, 0);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
