@@ -36,6 +36,7 @@
 #include "rb-source.h"
 #include "rb-util.h"
 #include "rb-static-playlist-source.h"
+#include "rb-plugin.h"
 
 static void rb_source_class_init (RBSourceClass *klass);
 static void rb_source_init (RBSource *source);
@@ -56,7 +57,6 @@ static gboolean default_can_rename (RBSource *source);
 static gboolean default_can_search (RBSource *source);
 static GList *default_copy (RBSource *source);
 static void default_reset_filters (RBSource *source);
-static void default_song_properties (RBSource *source);
 static gboolean default_try_playlist (RBSource *source);
 static RBSourceEOFType default_handle_eos (RBSource *source);
 static gboolean default_show_popup  (RBSource *source);
@@ -93,6 +93,7 @@ struct _RBSourcePrivate
 	guint update_visibility_id;
 	guint update_status_id;
 	RhythmDBEntryType entry_type;
+	RBPlugin *plugin;
 };
 
 enum
@@ -106,7 +107,8 @@ enum
 	PROP_QUERY_MODEL,
 	PROP_HIDDEN_WHEN_EMPTY,
 	PROP_SOURCELIST_GROUP,
-	PROP_ENTRY_TYPE
+	PROP_ENTRY_TYPE,
+	PROP_PLUGIN
 };
 
 enum
@@ -145,7 +147,6 @@ rb_source_class_init (RBSourceClass *klass)
 	klass->impl_get_entry_view = default_get_entry_view;
 	klass->impl_copy = default_copy;
 	klass->impl_reset_filters = default_reset_filters;
-	klass->impl_song_properties = default_song_properties;
 	klass->impl_handle_eos = default_handle_eos;
 	klass->impl_get_config_widget = NULL;
 	klass->impl_receive_drag = NULL;
@@ -236,6 +237,13 @@ rb_source_class_init (RBSourceClass *klass)
 							     "Type of the entries which should be displayed by this source",
 							     RHYTHMDB_TYPE_ENTRY_TYPE,
 							     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_PLUGIN,
+					 g_param_spec_object ("plugin",
+						 	      "RBPlugin",
+							      "RBPlugin instance for the plugin that created the source",
+							      RB_TYPE_PLUGIN,
+							      G_PARAM_READWRITE));
 
 	rb_source_signals[DELETED] =
 		g_signal_new ("deleted",
@@ -475,6 +483,9 @@ rb_source_set_property (GObject *object,
 	case PROP_ENTRY_TYPE:
 		priv->entry_type = g_value_get_boxed (value);
 		break;
+	case PROP_PLUGIN:
+		priv->plugin = g_value_get_object (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -521,6 +532,9 @@ rb_source_get_property (GObject *object,
 	case PROP_ENTRY_TYPE:
 		g_value_set_boxed (value, priv->entry_type);
 		break;
+	case PROP_PLUGIN:
+		g_value_set_object (value, priv->plugin);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -895,10 +909,12 @@ rb_source_reset_filters (RBSource *source)
 	klass->impl_reset_filters (source);
 }
 
-static void
-default_song_properties (RBSource *source)
+gboolean
+rb_source_can_show_properties (RBSource *source)
 {
-	g_assert_not_reached ();
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return (klass->impl_song_properties != NULL);
 }
 
 void
@@ -906,6 +922,7 @@ rb_source_song_properties (RBSource *source)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
+	g_assert (klass->impl_song_properties);
 	klass->impl_song_properties (source);
 }
 
