@@ -48,7 +48,6 @@ typedef enum {
 	RB_SOURCELIST_GROUP_LAST
 } RBSourceListGroup;
 
-
 GType rb_source_eof_type_get_type (void);
 #define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
@@ -77,7 +76,7 @@ struct _RBSource
 struct _RBSourceClass
 {
 	GtkHBoxClass parent;
-	
+
 	/* signals */
 	void (*status_changed)	(RBSource *source);
 	void (*filter_changed)	(RBSource *source);
@@ -108,7 +107,7 @@ struct _RBSourceClass
 	gboolean	(*impl_can_copy)	(RBSource *source);
 	gboolean	(*impl_can_paste)	(RBSource *source);
 	gboolean	(*impl_can_add_to_queue)(RBSource *source);
-	
+
 	GList *		(*impl_cut)		(RBSource *source);
 	GList *		(*impl_copy)		(RBSource *source);
 	void		(*impl_paste)		(RBSource *source, GList *entries);
@@ -119,14 +118,16 @@ struct _RBSourceClass
 	void		(*impl_song_properties)	(RBSource *source);
 
 	gboolean	(*impl_try_playlist)	(RBSource *source);
+	guint		(*impl_want_uri)	(RBSource *source, const char *uri);
+	gboolean	(*impl_add_uri)		(RBSource *source, const char *uri, const char *title, const char *genre);
 
 	gboolean	(*impl_can_pause)	(RBSource *source);
 	RBSourceEOFType	(*impl_handle_eos)	(RBSource *source);
-	
+
 	gboolean	(*impl_have_url)	(RBSource *source);
 	gboolean	(*impl_receive_drag)	(RBSource *source, GtkSelectionData *data);
 	gboolean	(*impl_show_popup)	(RBSource *source);
-				   
+
 	void		(*impl_delete_thyself)	(RBSource *source);
 	void		(*impl_activate)	(RBSource *source);
 	void		(*impl_deactivate)	(RBSource *source);
@@ -171,6 +172,7 @@ gboolean	rb_source_can_move_to_trash	(RBSource *source);
 gboolean	rb_source_can_copy		(RBSource *source);
 gboolean	rb_source_can_paste		(RBSource *source);
 gboolean	rb_source_can_add_to_queue	(RBSource *source);
+gboolean	rb_source_can_show_properties	(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
 GList *		rb_source_copy			(RBSource *source);
@@ -182,12 +184,12 @@ void		rb_source_move_to_trash		(RBSource *source);
 void		rb_source_song_properties	(RBSource *source);
 
 gboolean	rb_source_try_playlist		(RBSource *source);
+guint		rb_source_want_uri		(RBSource *source, const char *uri);
+gboolean	rb_source_add_uri		(RBSource *source, const char *uri, const char *title, const char *genre);
 
 gboolean	rb_source_can_pause		(RBSource *source);
 RBSourceEOFType	rb_source_handle_eos		(RBSource *source);
 
-gboolean	rb_source_have_url		(RBSource *source);
-
 gboolean	rb_source_receive_drag		(RBSource *source, GtkSelectionData *data);
 
 gboolean	rb_source_show_popup		(RBSource *source);
@@ -202,16 +204,17 @@ GList *		rb_source_get_search_actions	(RBSource *source);
 
 GList *		rb_source_gather_selected_properties (RBSource *source, RhythmDBPropType prop);
 
+void            rb_source_set_hidden_when_empty (RBSource *source,
+                                                 gboolean  hidden);
+
 /* Protected methods, should only be used by objects inheriting from RBSource */
-void            _rb_source_show_popup           (RBSource *source, 
+void            _rb_source_show_popup           (RBSource *source,
 						 const char *ui_path);
 GtkActionGroup *_rb_source_register_action_group (RBSource *source,
 						  const char *group_name,
 						  GtkActionEntry *actions,
 						  int num_actions,
 						  gpointer user_data);
-void		_rb_source_hide_when_empty	(RBSource *source,
-						 RhythmDBQueryModel *model);
 
 G_END_DECLS
 
