@@ -53,13 +53,12 @@ typedef struct
 
 GType		rb_header_get_type		(void);
 
-RBHeader *	rb_header_new			(RBShellPlayer *header);
+RBHeader *	rb_header_new			(RBShellPlayer *header,
+						 RhythmDB *db);
 
 void		rb_header_set_playing_entry	(RBHeader *header,
-						 RhythmDBEntry *entry);
-
-void		rb_header_set_title		(RBHeader *header,
-						 const char *title);
+						 RhythmDBEntry *entry,
+						 gboolean seekable);
 
 void		rb_header_set_show_position_slider (RBHeader *header,
                                                     gboolean show);
