@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of RhythmDB query creation dialog
  *
  *  Copyright (C) 2003, 2004 Colin Walters <walters@gnome.org>
@@ -19,10 +20,11 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 #include <stdlib.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "rhythmdb.h"
@@ -34,7 +36,6 @@
 #include "rb-glade-helpers.h"
 #include "rb-util.h"
 
-
 static void rb_query_creator_class_init (RBQueryCreatorClass *klass);
 static GObject *rb_query_creator_constructor (GType type, guint n_construct_properties,
 					      GObjectConstructParam *construct_properties);
@@ -75,7 +76,7 @@ typedef struct
 	RhythmDB *db;
 
 	gboolean creating;
-	
+
 	GtkSizeGroup *property_size_group;
 	GtkSizeGroup *criteria_size_group;
 	GtkSizeGroup *entry_size_group;
@@ -94,7 +95,6 @@ typedef struct
 	GtkWidget *sort_desc;
 } RBQueryCreatorPrivate;
 
-
 G_DEFINE_TYPE (RBQueryCreator, rb_query_creator, GTK_TYPE_DIALOG)
 #define QUERY_CREATOR_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), rb_query_creator_get_type(), RBQueryCreatorPrivate))
 
@@ -105,7 +105,6 @@ enum
 	PROP_CREATING,
 };
 
-
 static void
 rb_query_creator_class_init (RBQueryCreatorClass *klass)
 {
@@ -142,7 +141,8 @@ rb_query_creator_init (RBQueryCreator *creator)
 }
 
 static GObject *
-rb_query_creator_constructor (GType type, guint n_construct_properties,
+rb_query_creator_constructor (GType type,
+                              guint n_construct_properties,
 			      GObjectConstructParam *construct_properties)
 {
 	RBQueryCreatorPrivate *priv;
@@ -153,8 +153,8 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	creator = RB_QUERY_CREATOR (G_OBJECT_CLASS (rb_query_creator_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
 	priv = QUERY_CREATOR_GET_PRIVATE (creator);
-	
-	if (priv->creating) { 
+
+	if (priv->creating) {
 		gtk_dialog_add_button (GTK_DIALOG (creator),
 				       GTK_STOCK_CANCEL,
 				       GTK_RESPONSE_CLOSE);
@@ -186,7 +186,7 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	xml = rb_glade_xml_new ("create-playlist.glade",
 				"main_vbox",
 				creator);
-	
+
 	priv->disjunction_check = GTK_WIDGET (glade_xml_get_widget (xml, "disjunctionCheck"));
 	priv->limit_check = GTK_WIDGET (glade_xml_get_widget (xml, "limitCheck"));
 	priv->limit_entry = GTK_WIDGET (glade_xml_get_widget (xml, "limitEntry"));
@@ -203,7 +203,7 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	gtk_size_group_add_widget (priv->button_size_group, priv->addbutton);
 	g_signal_connect_object (G_OBJECT (priv->addbutton), "clicked", G_CALLBACK (add_button_click_cb),
 				 creator, 0);
-	
+
 	setup_sort_option_menu (creator, priv->sort_menu, sort_options, num_sort_options);
 
 	priv->vbox = GTK_BOX (glade_xml_get_widget (xml, "sub_vbox"));
@@ -214,6 +214,8 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (creator)->vbox), mainbox, FALSE, FALSE, 0);
 	gtk_widget_show_all (GTK_WIDGET (creator));
 
+	g_object_unref (xml);
+
 	return G_OBJECT (creator);
 }
 
@@ -243,7 +245,7 @@ rb_query_creator_dispose (GObject *object)
 	if (priv->rows)
 		g_list_free (priv->rows);
 	priv->rows = NULL;
-	
+
 	G_OBJECT_CLASS (rb_query_creator_parent_class)->dispose (object);
 }
 
@@ -270,11 +272,11 @@ rb_query_creator_set_property (GObject *object,
 	}
 }
 
-static void 
+static void
 rb_query_creator_get_property (GObject *object,
-			      guint prop_id,
-			      GValue *value,
-			      GParamSpec *pspec)
+                               guint prop_id,
+                               GValue *value,
+                               GParamSpec *pspec)
 {
 	RBQueryCreator *creator = RB_QUERY_CREATOR (object);
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
@@ -293,7 +295,6 @@ rb_query_creator_get_property (GObject *object,
 	}
 }
 
-
 GtkWidget *
 rb_query_creator_new (RhythmDB *db)
 {
@@ -301,8 +302,10 @@ rb_query_creator_new (RhythmDB *db)
 }
 
 static gboolean
-rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
-			     RhythmDBQueryModelLimitType limit_type, GValueArray *limit_value)
+rb_query_creator_load_query (RBQueryCreator *creator,
+                             GPtrArray *query,
+			     RhythmDBQueryModelLimitType limit_type,
+                             GValueArray *limit_value)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	int i;
@@ -324,7 +327,7 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 			if (data->type != RHYTHMDB_QUERY_DISJUNCTION)
 				append_row (creator);
 		}
-	
+
 	rows = priv->rows;
 
 	for (i = 0; i < subquery->len; i++) {
@@ -342,7 +345,7 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 		propmenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (rows->data), 0));
 		index = get_property_index_from_proptype (property_options, num_property_options, data->propid);
 		gtk_option_menu_set_history (propmenu, index);
-		
+
 		criteria_menu = get_box_widget_at_pos (GTK_BOX (rows->data), 1);
 		select_criteria_from_value (creator, criteria_menu, data->propid, data->type);
 
@@ -350,7 +353,7 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 		g_assert (property_type->criteria_set_widget_data != NULL);
 		property_type->criteria_set_widget_data (get_box_widget_at_pos (GTK_BOX (rows->data), 2),
 					      data->val);
-			
+
 		rows = rows->next;
 	}
 
@@ -401,7 +404,9 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 }
 
 static gboolean
-rb_query_creator_set_sorting (RBQueryCreator *creator, const char *sort_column, gint sort_direction)
+rb_query_creator_set_sorting (RBQueryCreator *creator,
+                              const char *sort_column,
+                              gint sort_direction)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	int i;
@@ -415,7 +420,7 @@ rb_query_creator_set_sorting (RBQueryCreator *creator, const char *sort_column,
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->sort_desc), (sort_direction == GTK_SORT_DESCENDING));
 
-	for (i = 0; i < num_property_options; i++)
+	for (i = 0; i < num_sort_options; i++)
 		if (strcmp (sort_options[i].sort_key, sort_column) == 0)
 			break;
 
@@ -424,13 +429,17 @@ rb_query_creator_set_sorting (RBQueryCreator *creator, const char *sort_column,
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU (priv->sort_menu), i);
 	sort_option_menu_changed (GTK_OPTION_MENU (priv->sort_menu), creator); /* force the checkbox to change label */
+
 	return TRUE;
 }
 
 GtkWidget *
-rb_query_creator_new_from_query (RhythmDB *db, GPtrArray *query,
-				 RhythmDBQueryModelLimitType limit_type, GValueArray *limit_value,
-				 const char *sort_column, gint sort_direction)
+rb_query_creator_new_from_query (RhythmDB *db,
+                                 GPtrArray *query,
+				 RhythmDBQueryModelLimitType limit_type,
+                                 GValueArray *limit_value,
+				 const char *sort_column,
+                                 gint sort_direction)
 {
 	RBQueryCreator *creator = g_object_new (RB_TYPE_QUERY_CREATOR, "db", db,
 						"creating", FALSE, NULL);
@@ -443,7 +452,6 @@ rb_query_creator_new_from_query (RhythmDB *db, GPtrArray *query,
 		return NULL;
 	}
 
-
 	return GTK_WIDGET (creator);
 }
 
@@ -469,7 +477,8 @@ get_box_widget_at_pos (GtkBox *box, guint pos)
 }
 
 static GtkWidget *
-get_entry_for_property (RBQueryCreator *creator, RhythmDBPropType prop,
+get_entry_for_property (RBQueryCreator *creator,
+                        RhythmDBPropType prop,
 			gboolean *constrain)
 {
 	const RBQueryCreatorPropertyType *property_type;
@@ -485,16 +494,20 @@ get_entry_for_property (RBQueryCreator *creator, RhythmDBPropType prop,
 GPtrArray *
 rb_query_creator_get_query (RBQueryCreator *creator)
 {
-	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
+	RBQueryCreatorPrivate *priv;
 	GPtrArray *query;
 	GPtrArray *sub_query;
 	GList *rows, *row;
 	gboolean disjunction;
 
+	g_return_val_if_fail (RB_IS_QUERY_CREATOR (creator), NULL);
+
+	priv = QUERY_CREATOR_GET_PRIVATE (creator);
+
 	disjunction = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->disjunction_check));
 	sub_query = g_ptr_array_new ();
 	rows = priv->rows;
-	
+
 	for (row = rows; row; row = row->next) {
 		GtkOptionMenu *propmenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (row->data),
 										  0));
@@ -504,12 +517,12 @@ rb_query_creator_get_query (RBQueryCreator *creator)
 		const RBQueryCreatorPropertyOption *prop_option = &property_options[prop_position];
 		const RBQueryCreatorCriteriaOption *criteria_options = prop_option->property_type->criteria_options;
 		const RBQueryCreatorCriteriaOption *criteria_option = &criteria_options[gtk_option_menu_get_history (criteria_menu)];
-			
+
 		g_assert (prop_option->property_type->criteria_get_widget_data != NULL);
 		{
 			RhythmDBQueryData *data = g_new0 (RhythmDBQueryData, 1);
 			GValue *val = g_new0 (GValue, 1);
-			
+
 			data->type = criteria_option->val;
 			data->propid = criteria_option->strict ? prop_option->strict_val : prop_option->fuzzy_val;
 
@@ -531,17 +544,22 @@ rb_query_creator_get_query (RBQueryCreator *creator)
 				      RHYTHMDB_PROP_TYPE,
 				      RHYTHMDB_ENTRY_TYPE_SONG,
 				      /* the constructed query */
-			       RHYTHMDB_QUERY_SUBQUERY,
-			       sub_query,
-			       RHYTHMDB_QUERY_END);
+                                      RHYTHMDB_QUERY_SUBQUERY,
+                                      sub_query,
+                                      RHYTHMDB_QUERY_END);
 	return query;
 }
 
 void
 rb_query_creator_get_limit (RBQueryCreator *creator,
-			    RhythmDBQueryModelLimitType *type, GValueArray **limit)
+			    RhythmDBQueryModelLimitType *type,
+                            GValueArray **limit)
 {
-	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
+	RBQueryCreatorPrivate *priv;
+
+	g_return_if_fail (RB_IS_QUERY_CREATOR (creator));
+
+	priv = QUERY_CREATOR_GET_PRIVATE (creator);
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->limit_check))) {
 		guint64 l;
@@ -579,22 +597,33 @@ rb_query_creator_get_limit (RBQueryCreator *creator,
 }
 
 void
-rb_query_creator_get_sort_order (RBQueryCreator *creator, const char **sort_column, gint *sort_direction)
+rb_query_creator_get_sort_order (RBQueryCreator *creator,
+                                 const char **sort_column,
+                                 gint *sort_direction)
 {
-	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
-	int i;
+	RBQueryCreatorPrivate *priv;
 
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->sort_desc)))
-		*sort_direction = GTK_SORT_DESCENDING;
-	else
-		*sort_direction = GTK_SORT_ASCENDING;
+	g_return_if_fail (RB_IS_QUERY_CREATOR (creator));
 
-	i = gtk_option_menu_get_history (GTK_OPTION_MENU (priv->sort_menu));
-	*sort_column = sort_options[i].sort_key;
+	priv = QUERY_CREATOR_GET_PRIVATE (creator);
+
+	if (sort_direction != NULL) {
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->sort_desc)))
+			*sort_direction = GTK_SORT_DESCENDING;
+		else
+			*sort_direction = GTK_SORT_ASCENDING;
+	}
+
+	if (sort_column != NULL) {
+		int i;
+		i = gtk_option_menu_get_history (GTK_OPTION_MENU (priv->sort_menu));
+		*sort_column = sort_options[i].sort_key;
+	}
 }
 
 static void
-limit_toggled_cb (GtkWidget *limit, RBQueryCreator *creator)
+limit_toggled_cb (GtkWidget *limit,
+                  RBQueryCreator *creator)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (limit));
@@ -607,7 +636,8 @@ limit_toggled_cb (GtkWidget *limit, RBQueryCreator *creator)
 }
 
 static GtkWidget *
-lookup_row_by_widget (RBQueryCreator *creator, GtkWidget *widget)
+lookup_row_by_widget (RBQueryCreator *creator,
+                      GtkWidget *widget)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	GList *rows = priv->rows;
@@ -628,7 +658,8 @@ lookup_row_by_widget (RBQueryCreator *creator, GtkWidget *widget)
 }
 
 static void
-remove_button_click_cb (GtkWidget *button, RBQueryCreator *creator)
+remove_button_click_cb (GtkWidget *button,
+                        RBQueryCreator *creator)
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	GtkWidget *row;
@@ -641,7 +672,8 @@ remove_button_click_cb (GtkWidget *button, RBQueryCreator *creator)
 }
 
 static void
-add_button_click_cb (GtkWidget *button, RBQueryCreator *creator)
+add_button_click_cb (GtkWidget *button,
+                     RBQueryCreator *creator)
 {
 	append_row (creator);
 }
@@ -694,10 +726,11 @@ append_row (RBQueryCreator *creator)
 
 static int
 get_property_index_from_proptype (const RBQueryCreatorPropertyOption *options,
-			    int length, RhythmDBPropType prop)
+                                  int length,
+                                  RhythmDBPropType prop)
 {
 	int i;
-	
+
 	for (i = 0; i < length; i++)
 		if (prop == options[i].strict_val || prop == options[i].fuzzy_val)
 			return i;
@@ -705,7 +738,6 @@ get_property_index_from_proptype (const RBQueryCreatorPropertyOption *options,
 	g_assert_not_reached ();
 }
 
-
 static void
 select_criteria_from_value (RBQueryCreator *creator,
 			    GtkWidget *option_menu,
@@ -719,7 +751,7 @@ select_criteria_from_value (RBQueryCreator *creator,
 	i = get_property_index_from_proptype (property_options, num_property_options, prop);
 	length = property_options[i].property_type->num_criteria_options;
 	options =  property_options[i].property_type->criteria_options;
-			
+
 	for (i = 0; i < length; i++) {
 		if (qtype == options[i].val) {
 			gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), i);
@@ -781,7 +813,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 
 static GtkWidget*
 create_property_option_menu (RBQueryCreator *creator,
-			    const RBQueryCreatorPropertyOption *options,
+                             const RBQueryCreatorPropertyOption *options,
 			     int length)
 {
 	GtkWidget *option_menu;
@@ -791,7 +823,7 @@ create_property_option_menu (RBQueryCreator *creator,
 
 	option_menu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
-    
+
 	/* add the property options */
 	for (i = 0; i < length; i++) {
 		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
@@ -807,7 +839,7 @@ create_property_option_menu (RBQueryCreator *creator,
 
 	g_signal_connect_object (G_OBJECT (option_menu), "changed",
 				 G_CALLBACK (property_option_menu_changed), creator, 0);
-  
+
 	return option_menu;
 }
 
@@ -822,7 +854,7 @@ create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
 
 	option_menu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
-    
+
 	for (i = 0; i < length; i++) {
 		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
@@ -831,7 +863,7 @@ create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
 	gtk_widget_show (menu);
-    
+
 	return option_menu;
 }
 
@@ -841,7 +873,7 @@ sort_option_menu_changed (GtkOptionMenu *propmenu,
 {
 	RBQueryCreatorPrivate *priv = QUERY_CREATOR_GET_PRIVATE (creator);
 	int index = gtk_option_menu_get_history (propmenu);
-  
+
 	gtk_button_set_label (GTK_BUTTON (priv->sort_desc), _(sort_options[index].sort_descending_name));
 	rb_debug("changing descending label to %s[%d]", sort_options[index].sort_descending_name, index);
 }
@@ -855,10 +887,10 @@ setup_sort_option_menu (RBQueryCreator *creator,
 	GtkWidget *menu;
 	GtkWidget *menu_item;
 	int i;
-  
+
 	menu = gtk_menu_new ();
 	gtk_widget_show (menu);
-    
+
 	for (i = 0; i < length; i++) {
 		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
@@ -867,6 +899,6 @@ setup_sort_option_menu (RBQueryCreator *creator,
 
 	g_signal_connect_object (G_OBJECT (option_menu), "changed",
 				 G_CALLBACK (sort_option_menu_changed), creator, 0);
-  
+
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
 }
